/*
 * p22_giveaway.c
 *
 *  Created on: 5 Mar 2019
 *      Author: ctorr
 */

// Runs in EMBEDDED MODE

// Application Identifier - used to select the application
#pragma attribute("aid","A00000014453444D")
// The name that will appear in the Directory entry
#pragma attribute("name","Secure Decision Maker")
// Permission to use peripherals (bit12)
#pragma attribute("access_list","1000")

// Standard MULTOS include file
#include <multos.h>

// Include file for Trust Anchor specific functions
#include <mi.h>

// Define GPIO pin that the switch is connected to
#define SWITCH_PIN	6

// Define GPIO pin that the LEDs are connected to
#define RED_LED_PIN 3
#define GREEN_LED_PIN 4

#pragma melsession
BYTE timerId;

// Exception handler
void
abendHandler(void)
{
	miPerformRestart();
}

void
main(void)
{
	BYTE id,info;
	BYTE resultPin;
	QWORD r;

	// Get event details
	id = miGetEmbeddedEventId();
	info = miGetEmbeddedEventInfo();

	// Initialisation is only required once at startup
	if(id == MI_EMBEDDED_EVENT_STARTUP)
	{
		// Trap exceptions (abends)
		miSetAbendFunction((WORD)abendHandler);

		// Allocate GPIO pin 5 (the one connected to the on-board switch)
		miGPIOAllocatePin(SWITCH_PIN);
		miGPIOEnablePullupPin(SWITCH_PIN);

		// Register interest in an event on GPIO port 0 (pins GPIO0 -> GPIO7)...
		miRegisterEmbeddedEvent(MI_EMBEDDED_EVENT_GPIO,0);

		// ... and monitor the switch
		miGPIOMonitorPin (SWITCH_PIN, MI_GPIO_EVENT_HIGH_TO_LOW);

		// Allocate LED pins and switch them off
		miGPIOAllocatePin(RED_LED_PIN);
		miGPIOEnableOutputPin(RED_LED_PIN);
		miGPIOAllocatePin(GREEN_LED_PIN);
		miGPIOEnableOutputPin(GREEN_LED_PIN);
		miGPIOSetPin(RED_LED_PIN,0);
		miGPIOSetPin(GREEN_LED_PIN,0);

		// Set up a timer to be used to put the chip back to sleep
		timerId = miTimerCreate(5000,MI_TIMER_TIME_UNIT_MILLISECONDS);
		miRegisterEmbeddedEvent(MI_EMBEDDED_EVENT_TIMER,timerId);

		// Go into ultra low power mode. Current consumption will be tiny until woken up by the GPIO event.
		// Note: In ULTRA_LOW mode, timers don't work so you must use another event type to wake up the chip.
		miSetEmbeddedPowerMode(MI_EMBEDDED_POWER_MODE_ULTRA_LOW);
	}

	// If there has been an event on GPIO port 0
	else if (id == MI_EMBEDDED_EVENT_GPIO && info == 0)
	{
		// Switch to normal power mode
		miSetEmbeddedPowerMode(MI_EMBEDDED_POWER_MODE_NORMAL);

		// See if the switch pin has been triggered
		if(miGPIOMonitoredPinEventOccurred(SWITCH_PIN))
		{
			// Randomly select answer
			r = multosGetRandomNumber();
			if(r.lsdw & 0x00000001)
				resultPin = RED_LED_PIN;
			else
				resultPin = GREEN_LED_PIN;

			// Flash that LED for a second
			miGPIOSetPin(resultPin,MI_GPIO_PIN_STATE_HIGH);
			miDelay(1000);
			miGPIOSetPin(resultPin,MI_GPIO_PIN_STATE_LOW);
		}

		// Because of switch bounce, many events can be generated when pushing the switch. To clear those
		// unwanted events, the event queue should be flushed as follows.
		miFlushEmbeddedEventWithInfo(id,info);

		// Leave chip active for a bit longer for NFC to pick it up.
		miTimerStart(timerId);
	}
	else if (id == MI_EMBEDDED_EVENT_TIMER && info == timerId)
	{
		// Go back into ultra low power mode
		miSetEmbeddedPowerMode(MI_EMBEDDED_POWER_MODE_ULTRA_LOW);
	}
	// On exit, control returns to MULTOS for it to continue processing events.
}
